#include <string.h>
#include <assert.h>
#include <stdarg.h>
#include <stdlib.h>
#include <dialog.h>
#include <proto.h>
#include <diajava.h>
#include "framework.h"
#include "framework.m"
#include <map>
#include <string>
#include <tlmpdia.h>

using namespace std;

#define NB_AREA	5

class FRAMEWORK_MSGS_F: public FRAMEWORK_MSGS{
	class _F_FRAMEWORK &c;
	/*~PROTOBEG~ FRAMEWORK_MSGS_F */
public:
	FRAMEWORK_MSGS_F (_F_FRAMEWORK&_c);
	void endok (void);
	/*~PROTOEND~ FRAMEWORK_MSGS_F */
};



PUBLIC FRAMEWORK_MSGS_F::FRAMEWORK_MSGS_F(_F_FRAMEWORK &_c)
	: c(_c)
{
}

PUBLIC void FRAMEWORK_MSGS_F::endok()
{
	c.endok();
}

PUBLIC FRAMEWORK_INFO::FRAMEWORK_INFO(
	FRAMEWORK_MSGS &_msgs)
	: msgs(_msgs)
{
	has_focus = false;
	uithread_id = ::uithread_id;
}

PUBLIC FRAMEWORK_INFO *FRAMEWORK_INFOS::getitem (int no) const
{
	return (FRAMEWORK_INFO*)ARRAY::getitem(no);
}

class FRAMEWORK_PRIVATE{
public:
	SSTRING title;
	int menulevel;
	bool menubar_done;
	bool taskbar_done;
	int taskid;
	int menuid;
	SSTRING mainid;			// ID of the mainform, including the parent context
							// if needed.
	SSTRING ctx_status;
	FRAMEWORK_INFO *parent;
	PRIVATE_MESSAGE *msg;
	PRIVATE_MESSAGE enddocument;	// Sent when a document ended
	PRIVATE_MESSAGE startdocument;	// Sent when the document is starting
									// this way _F_framework::newdocument
									// only returns when the new document
									// has started
	FRAMEWORK_MSGS_F controlmsgs;
	PRIVATE_MESSAGE requestend;	// Sent when one document request the framework
								// to end.
	PRIVATE_MESSAGE manageend;	// Sent when manage() is done
	PRIVATE_MESSAGE allmayend;	// Message sent when all document accepted to end
	int nbdocument[NB_AREA];	// Number of document displayed currently
	int nbmayend;				// How many document have called the endok() function
	SSTRINGS layoutids;
	SSTRING ctxarea[NB_AREA];	// Context to reach the form
	bool area_isbook[NB_AREA];
	int idalloc;				// Int to allocate id for layout components
								// see pushid()
	FRAMEWORK_INFOS infos[6];	// Remember the document for each
								// areas
	bool end_was_called;
	bool parent_request_end;	// The parent framework request us to end
								// so we have to confirm if we can.
								// After the parent has confirm with
								// all child, it will tell us to end.
	bool all_ending;			// Ok, every document is ending now
								// since they all agree to do so
	map<int,string> docids;		// IDs of the active document
	ARRAY_OBJS<FRAMEWORK_DOCUMENT> docs;	// Active document
	/*~PROTOBEG~ FRAMEWORK_PRIVATE */
public:
	FRAMEWORK_PRIVATE (_F_FRAMEWORK&_c);
	void adddoc (FRAMEWORK_DOCUMENT *doc);
	void deldoc (FRAMEWORK_DOCUMENT *doc);
	FRAMEWORK_INFO *getinfo (void);
	int getnbdocument (void);
	void initfocus (FRAMEWORK_INFO *focus[6]);
	bool knowndoc (FRAMEWORK_DOCUMENT *doc);
	/*~PROTOEND~ FRAMEWORK_PRIVATE */
};

PUBLIC void FRAMEWORK_PRIVATE::adddoc (FRAMEWORK_DOCUMENT *doc)
{
	docs.add (doc);
}
PUBLIC void FRAMEWORK_PRIVATE::deldoc (FRAMEWORK_DOCUMENT *doc)
{
	docs.remove_del (doc);
}
/*
	Check if we already have an active document corresponding to that one.
*/
PUBLIC bool FRAMEWORK_PRIVATE::knowndoc (FRAMEWORK_DOCUMENT *doc)
{
	bool ret = false;
	for (int i=0; i<docs.getnb(); i++){
		FRAMEWORK_DOCUMENT *d = docs.getitem(i);
		if (d->same(*doc)){
			ret = true;
			break;
		}
	}
	return ret;
}

PUBLIC FRAMEWORK_PRIVATE::FRAMEWORK_PRIVATE(_F_FRAMEWORK &_c)
	: controlmsgs(_c)
{
	dialog_noquitbutton();
	menulevel = 0;
	menubar_done = false;
	taskbar_done = false;
	memset (nbdocument,0,sizeof(nbdocument));
	memset (area_isbook,0,sizeof(area_isbook));
	nbmayend = 0;
	parent = NULL;
	idalloc = 0;
	for (int i=0; i<6; i++) infos[i].neverdelete();
	all_ending = false;
	parent_request_end = false;
	end_was_called = false;
}

PUBLIC int FRAMEWORK_PRIVATE::getnbdocument()
{
	int ret = 0;
	for (int i=0; i<NB_AREA; i++){
		ret += nbdocument[i];
	}
	return ret;
}

/*
	Initialise focus[] so it contains the pointer to the visible
	document in each area.
*/
PUBLIC void FRAMEWORK_PRIVATE::initfocus (FRAMEWORK_INFO *focus[6])
{
	for (int i=0; i<6; i++){
		focus[i] = NULL;
		FRAMEWORK_INFOS *ifs = infos+i;
		for (int j=0; j<ifs->getnb(); j++){
			FRAMEWORK_INFO *info = ifs->getitem(j);
			if (info->has_focus){
				focus[i] = info;
				break;
			}
		}
	}
}

/*
	Retrieve the FRAMEWORK_INFO associated with the current thread
*/
PUBLIC FRAMEWORK_INFO *FRAMEWORK_PRIVATE::getinfo()
{
	FRAMEWORK_INFO *ret = NULL;
	for (int i=0; i<6; i++){
		FRAMEWORK_INFOS *ifs = infos+i;
		for (int j=0; j<ifs->getnb(); j++){
			FRAMEWORK_INFO *info = ifs->getitem(j);
			if (info->uithread_id == uithread_id){
				ret = info;
				break;
			}
		}
	}
	assert (ret != NULL);
	return ret;
}

void _F_FRAMEWORK::topmenu (const char *title)
{
	if (!priv->menubar_done){
		priv->menubar_done = true;
		diagui_sendcmd (P_Menubar,"main\n");
	}
	for (int i=priv->menulevel; i>0; i--) diagui_sendcmd (P_End,"\n");
	priv->menulevel = 0;
	submenu (title);
}
void _F_FRAMEWORK::helpmenu ()
{
	if (!priv->menubar_done){
		priv->menubar_done = true;
		diagui_sendcmd (P_Menubar,"main\n");
	}
	for (int i=priv->menulevel; i>0; i--) diagui_sendcmd (P_End,"\n");
	priv->menulevel = 0;
	submenu (MSG_U(M_HELP,"Help"));
	menuentry (10001,MSG_U(M_HELPABOUT,"About"));
	menuentry (10002,MSG_U(M_HELPINDEX,"Index"));
}
void _F_FRAMEWORK::menuentry (int id, const char *title)
{
	if (!priv->menubar_done){
		topmenu (MSG_U(M_DUMMY,"Dummy"));
		fprintf (stderr,"menuentry called before topmenu\n");
	}
	char tmp[1000];
	diagui_sendcmd (P_Menuentry,"%d %s\n",id,diagui_quote(title,tmp));
}
void _F_FRAMEWORK::submenu (const char *title)
{
	if (!priv->menubar_done){
		topmenu (MSG_R(M_DUMMY));
		fprintf (stderr,"submenu called before topmenu\n");
	}
	char tmp[1000];
	diagui_sendcmd (P_Submenu,"%s\n",diagui_quote(title,tmp));
	priv->menulevel++;
}
void _F_FRAMEWORK::endmenu ()
{
	if (priv->menulevel > 0){
		diagui_sendcmd (P_End,"\n");
		priv->menulevel--;
	}else{
		fprintf (stderr,"Too many framework::endmenu()\n");
	}
}
void _F_FRAMEWORK::newbutton(
	int id,
	const char *icon,
	const char *title,
	const char *help)
{
	if (!priv->taskbar_done){
		priv->taskbar_done = true;
		diagui_sendcmd (P_Form,"taskbar\n");
	}
	if (icon != NULL && icon[0] != '\0'){
		char tmpicon[100];
		diagui_sendminixpm (icon,tmpicon);
		char tmp1[1000],tmp2[1000];
		diagui_sendcmd (P_Button_xpm,"%d %s %s\n",id
			,diagui_quote(tmpicon,tmp1)
			,diagui_quote(title,tmp2));
	}else{
		char tmp2[1000];
		diagui_sendcmd (P_Button,"%d 0 %s\n",id
			,diagui_quote(title,tmp2));
	}
}
/*
	Check if some messages were received
*/
void _F_FRAMEWORK::messages()
{
}
/*
	Called when the framework is ending, after having request permission
	to end, before asking everyone to end.
*/
void _F_FRAMEWORK::ending (bool &)
{
}


/*
	Setup the task bar
*/
void _F_FRAMEWORK::taskbar()
{
}
/*
	Setup the menu bar
*/
void _F_FRAMEWORK::menubar()
{
}
/*
	execute one command from the menubar
*/
void _F_FRAMEWORK::menudo(int , FRAMEWORK_INFOS [], FRAMEWORK_INFO *[])
{
}
/*
	Execute one command from the task bar
*/
void _F_FRAMEWORK::taskdo(int , FRAMEWORK_INFOS [], FRAMEWORK_INFO *[])
{
}

static void f_taskdo(void *p)
{
	_F_FRAMEWORK *c = (_F_FRAMEWORK*)p;
	diagui_setdefaultctx (NULL);
	FRAMEWORK_INFO *focus[6];
	c->priv->initfocus (focus);
	c->taskdo(c->priv->taskid,c->priv->infos,focus);
}

static void f_menudo(void *p)
{
	_F_FRAMEWORK *c = (_F_FRAMEWORK*)p;
	diagui_setdefaultctx (NULL);
	FRAMEWORK_INFO *focus[6];
	c->priv->initfocus (focus);
	c->menudo(c->priv->menuid,c->priv->infos,focus);
}
static void f_helpindex(void *p)
{
	_F_FRAMEWORK *c = (_F_FRAMEWORK*)p;
	diagui_setdefaultctx (NULL);
	c->helpindex();
}
struct FRAMEWORK_DATA{
	_F_FRAMEWORK *c;
	void *data;
	int noarea;
	bool isfdoc;		// Is data a FRAMEWORK_DOCUMENT
};

void _F_FRAMEWORK::call_area (int noarea, void *data)
{
	const char *graphical_context = priv->ctxarea[noarea-1].get();
	diagui_setdefaultctx (graphical_context);

	FRAMEWORK_INFO info(priv->controlmsgs);
	int no = noarea-1;
	info.nbdocument = priv->nbdocument[no];
	priv->infos[no].add (&info);
	info.graphical_context = graphical_context;
	info.data = data;
	info.c = this;
	info.noarea = noarea;

	switch (noarea){
	case 1:
		area1 (info);
		break;
	case 2:
		area2 (info);
		break;
	case 3:
		area3 (info);
		break;
	case 4:
		area4 (info);
		break;
	case 5:
		area5 (info);
		break;
	}
	priv->infos[no].remove (&info);
}

void _F_FRAMEWORK::area1 (FRAMEWORK_INFO &info)
{
	document (1,info);
}
void _F_FRAMEWORK::area2 (FRAMEWORK_INFO &info)
{
	document (2,info);
}
void _F_FRAMEWORK::area3 (FRAMEWORK_INFO &info)
{
	document (3,info);
}
void _F_FRAMEWORK::area4 (FRAMEWORK_INFO &info)
{
	document (4,info);
}
void _F_FRAMEWORK::area5 (FRAMEWORK_INFO &info)
{
	document (5,info);
}

void _F_FRAMEWORK::document (
	int noarea,
	FRAMEWORK_INFO &info)
{
}


static void f_newdocument(void *p)
{
	FRAMEWORK_DATA *d = (FRAMEWORK_DATA*)p;
	_F_FRAMEWORK *c = d->c;
	int noarea = d->noarea-1;
	bool isfdoc = d->isfdoc;
	void *data = d->data;
	c->priv->nbdocument[noarea]++;
	// fprintf (stderr,"Sending startdocument\n");
	dialog_sendmessage (c->priv->startdocument);
	c->call_area (noarea+1,data);
	c->priv->nbdocument[noarea]--;
	dialog_sendmessage (c->priv->enddocument);
	if (isfdoc) c->priv->deldoc((FRAMEWORK_DOCUMENT*)data);
	if (!c->priv->all_ending) c->priv->docids[uithread_id] = "";
}

PUBLIC int FRAMEWORK::getdocumentids(SSTRINGS &tb)
{
	int ret = 0;
	map<int,string>::iterator it = c->priv->docids.begin();
	while (it != c->priv->docids.end()){
		if (it->second != ""){
			tb.add (it->second.c_str());
			ret ++;
		}
		it++;
	}
	return ret;
}

/*
	Trigger a new document in some area of the framework
*/
static void framework_newdocument(
	FRAMEWORK_PRIVATE *priv,
	_F_FRAMEWORK *c,
	void *data,
	int noarea,
	bool isfdoc)
{
	FRAMEWORK_DATA d;
	d.data = data;
	d.c = c;
	d.noarea = noarea;
	d.isfdoc = isfdoc;
	if (uithread (f_newdocument,&d) != -1){
		// We want the document function to start immediatly before
		// returning to the caller. This allows the caller to start
		// several documents at once
		// Normally a new uithread is started only when the caller
		// has yielded. For now, there is no uithread_yield() function.
		// We simulate this using PRIVATE_MESSAGE.
		//uithread_yield();
		dialog_waitformessage (priv->startdocument);
	}
}

/*
	Trigger a new document in some area of the framework
*/
void _F_FRAMEWORK::newdocument(void *data, int noarea)
{
	framework_newdocument (priv,this,data,noarea,false);
}

/*
	Trigger a new document in the area 2 of the framework
*/
void _F_FRAMEWORK::newdocument (void *data)
{
	newdocument (data,1);
}

/*
	Trigger a new document in the area 2 of the framework
*/
void _F_FRAMEWORK::newdocument ()
{
	newdocument ((void*)NULL,1);
}

void _F_FRAMEWORK::newdocument(FRAMEWORK_DOCUMENT *data, int noarea)
{
	// We check if this document is already active
	// If this is the case, we request this document to have focus
	if (priv->knowndoc(data)){
		// Controlling focus with the GUI is not working yet
	}else{
		priv->adddoc (data);
		framework_newdocument (priv,this,(void*)data,noarea,true);
	}
}
void _F_FRAMEWORK::newdocument(FRAMEWORK_DOCUMENT *data)
{
	newdocument (data,1);
}



void _F_FRAMEWORK::message()
{
}
void _F_FRAMEWORK::about()
{
	xconf_notice (MSG_U(N_NOABOUT,"No about screen defined"));
}
void _F_FRAMEWORK::helpindex()
{
	xconf_notice (MSG_U(N_NOHELPINDEX,"Help index not implemented yet"));
}

/*
	Display a default document (for example) when there is no
	document displayed anymore.

	Set end to true if the framework must end
*/
void _F_FRAMEWORK::nodocument(int, bool &end)
{
	end = true;
}

/*
	Use to display some status information at the bottom of the framework.
	The message is used to tell when the framework is ending.
*/
void _F_FRAMEWORK::status(PRIVATE_MESSAGE &)
{
}

static void f_status (void *p)
{
	_F_FRAMEWORK *c = (_F_FRAMEWORK*)p;
	diagui_setdefaultctx (c->priv->ctx_status.get());
	c->status(c->priv->controlmsgs.ending);
}


void _F_FRAMEWORK::end()
{
	priv->end_was_called = true;
	dialog_sendmessage (priv->requestend);
}

void _F_FRAMEWORK::endok()
{
	priv->nbmayend++;
	if (priv->getnbdocument() == priv->nbmayend){
		dialog_sendmessage (priv->allmayend);
	}
}

void _F_FRAMEWORK::layout_area (int noarea, bool isbook)
{
	char id[10];
	sprintf (id,"area%d",noarea);
	SSTRING *s = priv->layoutids.getitem(priv->layoutids.getnb()-1);
	SSTRING *ctx = priv->ctxarea+noarea;
	priv->area_isbook[noarea] = isbook;
	if (s != NULL){
		ctx->setfromf ("%s.%s.%s",priv->mainid.get(),s->get(),id);
	}else{
		ctx->setfromf ("%s.%s",priv->mainid.get(),id);
	}
	if (isbook){
		diagui_sendcmd (P_Book,"%s $focus=B%d\n",id,200+noarea);
	}else{
		diagui_sendcmd (P_Form,"%s\n",id);
	}
	diagui_sendcmd (P_End,"\n");
}

void _F_FRAMEWORK::layout_area1(bool isbook)
{
	layout_area (0,isbook);
}
void _F_FRAMEWORK::layout_area2(bool isbook)
{
	layout_area (1,isbook);
}
void _F_FRAMEWORK::layout_area3(bool isbook)
{
	layout_area (2,isbook);
}
void _F_FRAMEWORK::layout_area4(bool isbook)
{
	layout_area (3,isbook);
}
void _F_FRAMEWORK::layout_area5(bool isbook)
{
	layout_area (4,isbook);
}

void _F_FRAMEWORK::layout_end()
{
	priv->layoutids.remove_del(priv->layoutids.getnb()-1);
	diagui_sendcmd (P_End,"\n");
}
void _F_FRAMEWORK::pushid (char id[10])
{
	sprintf (id,"f%d",priv->idalloc++);
	SSTRING *s = priv->layoutids.getitem(priv->layoutids.getnb()-1);
	SSTRING *ss = new SSTRING;
	if (s == NULL){
		ss->setfrom (id);
	}else{
		ss->setfromf ("%s.%s",s->get(),id);
	}
	priv->layoutids.add (ss);
}
void _F_FRAMEWORK::layout_form()
{
	char id[10];
	pushid (id);
	diagui_sendcmd (P_Form,"%s\n",id);
}
void _F_FRAMEWORK::layout_group(const char *title)
{
	char id[10];
	pushid (id);
	char tmp[1000];
	diagui_sendcmd (P_Group,"%s %s\n",id,diagui_quote(title,tmp));
}

void _F_FRAMEWORK::layout_book ()
{
	char id[10];
	pushid (id);
	diagui_sendcmd (P_Book,"%s\n",id);
}

void _F_FRAMEWORK::layout_page (const char *title)
{
	char id[10];
	pushid (id);
	char tmp[1000];
	diagui_sendcmd (P_Page,"%s %s\n",id,diagui_quote(title,tmp));	
}

void _F_FRAMEWORK::layout_newline()
{
	diagui_sendcmd (P_Newline,"\n");
}
void _F_FRAMEWORK::layout_dispolast(
	GUI_H_DISPO hdisp,
	int nbhcell,
	GUI_V_DISPO vdisp,
	int nbvcell)
{
	static char tbh[]={'L','C','R'};
	static char tbv[]={'T','C','B'};
	diagui_sendcmd (P_Dispolast,"%c %d %c %d\n",tbh[hdisp],nbhcell
		,tbv[vdisp],nbvcell);
}

void _F_FRAMEWORK::layout_cmd(int cmd, const char *ctl, ...)
{
}


void _F_FRAMEWORK::layout()
{
	// layout_area1(false);
	layout_area1(true);
}

/*
	Simple case to handle the control messages. This function
	is used by a document which always accept to end.
	The function return true when the document must end.
*/
bool _F_FRAMEWORK::must_end(FRAMEWORK_MSGS &msgs)
{
	bool ret = false;
	if (msgs.is_mayend()){
		endok();
	}else if (msgs.is_ending()){
		ret = true;
	}
	return ret;
}

bool _F_FRAMEWORK::must_end()
{
	FRAMEWORK_INFO *info = priv->getinfo();
	return must_end (info->msgs);
}
/*
	Make sure the dialog waits for all useful message
*/
void _F_FRAMEWORK::waitforall(DIALOG &dia)
{
	FRAMEWORK_INFO *info = priv->getinfo();
	dia.waitfor (info->docmsgs.menubar);
	dia.waitfor (info->docmsgs.taskbar);
	dia.waitfor (info->docmsgs.focus);
	dia.waitfor (info->docmsgs.close);
	dia.waitfor (info->docmsgs.save);
	dia.waitfor (info->docmsgs.saveas);
	info->msgs.waitfor (dia);
}

/*
	Return true if the menuid was actived in the menubar
*/
bool _F_FRAMEWORK::ismenuid(int menuid)
{
	FRAMEWORK_INFO *info = priv->getinfo();
	return dialog_testmessage(info->docmsgs.menubar)
		&& info->menuid == menuid;
}
/*
	Return true if the user has requested to close this document
	by closing the notebook view (button 2 in notebook pad for now)
*/
bool _F_FRAMEWORK::isclose()
{
	FRAMEWORK_INFO *info = priv->getinfo();
	UISTATE st;
	diajava_lastmousestate (st);
	return dialog_testmessage(info->docmsgs.focus)
		&& st.middleb;
}

/*
	Return true if the taskid was actived in the taskbar
*/
bool _F_FRAMEWORK::istaskid(int taskid)
{
	FRAMEWORK_INFO *info = priv->getinfo();
	return dialog_testmessage(info->docmsgs.taskbar)
		&& info->taskid == taskid;
}


/*
	Record the name of the document, useful to know which document are
	active at any given time.

	If the framework is ordered to end, it will remember which document
	were active at this time and the function FRAMEWORK::getactivedoc()
	will return the list of active document (Even if they are dead at this
	point).
*/
void _F_FRAMEWORK::setdocumentid(const char *s, ...)
{
	char tmp[1000];
	va_list list;
	va_start (list,s);
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	va_end (list);
	priv->docids[uithread_id] = tmp;
}

/*
	Send a menubar or taskbar event to all document having focus
*/
PRIVATE void FRAMEWORK::sendmsg2focus(int menuid, int taskid)
{
	FRAMEWORK_PRIVATE *priv = c->priv;
	for (int i=0; i<6; i++){
		FRAMEWORK_INFOS *ifs = priv->infos+i;
		for (int j=0; j<ifs->getnb(); j++){
			FRAMEWORK_INFO *info = ifs->getitem(j);
			if (info->has_focus){
				info->menuid = menuid;
				info->taskid = taskid;
				if (menuid != 0){
					dialog_sendmessage (info->docmsgs.menubar);
				}
				if (taskid != 0){
					dialog_sendmessage (info->docmsgs.taskbar);
				}
			}
		}
	}
}


PUBLIC void FRAMEWORK::manage ()
{
	FRAMEWORK_PRIVATE *priv = c->priv;
	if (dialog_mode != DIALOG_GUI){
		xconf_notice (MSG_U(E_GUIONLY
			,"This program only works in GUI mode"));
	}else{
		char tmp[1000];
		char mainid[10];
		sprintf (mainid,"main-%d",uithread_id);
		if (priv->parent != NULL){
			const char *ctx = priv->parent->graphical_context;
			priv->mainid.setfromf("%s.%s",ctx,mainid);
			diagui_sendcmd (P_Setcontext,"%s\n",ctx);
		}else{
			priv->mainid.setfrom(mainid);
		}
		priv->ctx_status.setfromf("%s.status",priv->mainid.get());
		diagui_sendcmd (P_MainForm,"%s %s $htrigger=1000\n",mainid
			,diagui_quote(priv->title.get(),tmp));
		c->menubar();
		if (priv->menubar_done){
			for (int i=priv->menulevel; i>=0; i--) diagui_sendcmd (P_End,"\n");
		}
		c->taskbar();
		if (priv->taskbar_done){
			diagui_sendcmd (P_End,"\n");
			diagui_sendcmd (P_Dispolast,"l 10 c 1\n");
			diagui_sendcmd (P_Newline,"\n");
		}
		c->layout();
		diagui_sendcmd (P_Newline,"\n");
		diagui_sendcmd (P_Form,"status\n");
		diagui_sendcmd (P_End,"\n");
		diagui_sendcmd (P_Dispolast,"l 10 c 1\n");
		diagui_sendcmd (P_End,"$nopopup=1\n");
		if (priv->parent){
			diagui_sendcmd (P_End,"\n");
		}

		for (int i=0; i<NB_AREA; i++){
			if (priv->ctxarea[i].is_filled()
				&& !priv->area_isbook[i]) newdocument ((void*)NULL,i+1);
		}
		uithread (f_status,c);
		while (1){
			SSTRING path, menu, action;
			int code = diagui_sync (mainid,path, action, menu);
			// fprintf (stderr,"framework[%d] code=%d :%s:%s:%s:\n",uithread_id,code,path.get(),menu.get(),action.get());
			if (dialog_testmessage(priv->enddocument)
				&& priv->getnbdocument() == 0){
				// Ok, all document have exited
				// either because we have requested them to do so
				// (priv->nbmayend > 0) or because the user close
				// the various documents one by one
				if (priv->nbmayend > 0){
					break;
				}else{
					bool end = false;
					c->nodocument(2,end);
					if (end) break;
				}
			}else if (code == 99
				|| dialog_testmessage(priv->requestend)){
				if (priv->getnbdocument() == 0){
					break;
				}else{
					// Ask all documents if they accept to end
					priv->parent_request_end = false;
					priv->nbmayend = 0;
					dialog_sendmessage (priv->controlmsgs.mayend);
				}
			}else if (code >= 200 && code < 206){
				FRAMEWORK_INFOS *ifs = priv->infos + code - 200;
				for (int i=0; i<ifs->getnb(); i++){
					ifs->getitem(i)->has_focus = false;
				}
				int nodoc = atoi(diajava_getextrareport());
				FRAMEWORK_INFO *info = ifs->getitem(nodoc);
				// fprintf (stderr,"setfocus this=%p nodoc=%d %p\n",this,nodoc,info);
				if (info != NULL){
					info->has_focus = true;
					dialog_sendmessage(info->docmsgs.focus);
				}
			}else if (dialog_testmessage(priv->allmayend)){
				if (priv->parent != NULL && priv->parent_request_end){
					// Tell the parent we accept to end
					priv->parent->c->endok();
				}else{
					bool do_not_end = false;
					c->ending(do_not_end);
					if (!do_not_end){
						priv->all_ending = true;
						dialog_sendmessage (priv->controlmsgs.ending);
					}
				}
			}else if (priv->msg != NULL && dialog_testmessage(*priv->msg)){
				c->message();
			}else if (priv->parent != NULL
				&& dialog_testmessage(priv->parent->msgs.ending)){
				bool do_not_end = false;
				c->ending(do_not_end);
				if (!do_not_end){
					priv->all_ending = true;
					dialog_sendmessage (priv->controlmsgs.ending);
				}
			}else if (priv->parent != NULL
				&& dialog_testmessage(priv->parent->msgs.mayend)){
				priv->parent_request_end = true;
				priv->nbmayend = 0;
				dialog_sendmessage (priv->controlmsgs.mayend);
			}else if (!menu.is_empty()){
				priv->menuid = menu.getval();
				if (priv->menuid == 10001){
					c->about();
				}else if (priv->menuid == 10002){
					uithread (f_helpindex,c);
				}else{
					uithread (f_menudo,c);
					for (int i=0; i<6; i++){
						FRAMEWORK_INFOS *ifs = priv->infos+i;
						for (int j=0; j<ifs->getnb(); j++){
							FRAMEWORK_INFO *info = ifs->getitem(j);
							if (info->has_focus){
								info->menuid = priv->menuid;
								dialog_sendmessage (info->docmsgs.menubar);
							}
						}
					}
				}
			}else if (priv->parent != NULL
				&& dialog_testmessage(priv->parent->docmsgs.taskbar)){
				priv->taskid = priv->parent->taskid;
				uithread (f_taskdo,c);
				sendmsg2focus(0,priv->taskid);
			}else if (priv->parent != NULL
				&& dialog_testmessage(priv->parent->docmsgs.menubar)){
				priv->menuid = priv->parent->menuid;
				uithread (f_menudo,c);
				sendmsg2focus(priv->menuid,0);
			}else if (action.is_filled()){
				priv->taskid = action.getval();
				uithread (f_taskdo,c);
				sendmsg2focus(0,priv->taskid);
			}else{
				priv->end_was_called = false;
				c->messages();
				if (priv->end_was_called){
					if (priv->getnbdocument() == 0){
						break;
					}else{
						// Ask all documents if they accept to end
						priv->parent_request_end = false;
						priv->nbmayend = 0;
						dialog_sendmessage (priv->controlmsgs.mayend);
					}
				}
			}
		}
		//fprintf (stderr,"Framework %d end\n",uithread_id);
		diagui_sendcmd (P_Delete,"%s\n",priv->mainid.get());
	}
	dialog_sendmessage (priv->manageend);
}

static void f_manage(void *p)
{
	FRAMEWORK *f = (FRAMEWORK*)p;
	f->manage ();

}

PRIVATE void FRAMEWORK::init(
	_F_FRAMEWORK &_c,
	const char *title,
	PRIVATE_MESSAGE *msg,	// Use to trigger the message function
	FRAMEWORK_INFO *info)
{
	dialog_clear();
	c = &_c;
	c->priv = new FRAMEWORK_PRIVATE(_c);
	c->priv->msg = msg;
	c->priv->title.setfrom (title);
	c->priv->parent = info;
	uithread (f_manage,this);
	uithread_yield();
}


PUBLIC FRAMEWORK::FRAMEWORK (
	_F_FRAMEWORK &_c,
	const char *title,
	PRIVATE_MESSAGE &msg)	// Use to trigger the message function
{
	init (_c,title,&msg,NULL);
}

PUBLIC FRAMEWORK::FRAMEWORK (
	_F_FRAMEWORK &_c,
	const char *title)
{
	init (_c,title,NULL,NULL);
}

PUBLIC FRAMEWORK::FRAMEWORK (
	_F_FRAMEWORK &_c,
	FRAMEWORK_INFO &info)
{
	init (_c,"",NULL,&info);
}

PUBLIC FRAMEWORK::FRAMEWORK (
	_F_FRAMEWORK &_c,
	const char *title,
	FRAMEWORK_INFO &info)
{
	init (_c,title,NULL,&info);
}

PUBLIC void FRAMEWORK::newdocument (void *p, int noarea)
{
	c->newdocument(p, noarea);
}

PUBLIC void FRAMEWORK::newdocument (void *p)
{
	newdocument(p,1);
}
PUBLIC void FRAMEWORK::newdocument ()
{
	newdocument((void*)NULL,1);
}
PUBLIC void FRAMEWORK::newdocument(FRAMEWORK_DOCUMENT *p, int noarea)
{
	c->newdocument(p, noarea);
}
PUBLIC void FRAMEWORK::newdocument(FRAMEWORK_DOCUMENT *p)
{
	newdocument (p,1);
}


PUBLIC void FRAMEWORK::loop ()
{
	diagui_sendcmd (P_Show,"%s 1\n",c->priv->mainid.get());
	dialog_waitformessage (c->priv->manageend);
}

/*
	This function return true if the user answer yes.
	When a user request a quit, potentially many area of the
	program (many documents) may ask "Quit without save" all at once
	(they all received the message "mayend").

	This function will popup one query at a time. If the user answer
	no to one, all the other will return false without showing
	their query.
*/
bool framework_quitok (const char *title, const char *intro)
{
	bool ret = false;
	static bool in_use = false;
	static bool global_ret = false;
	static PRIVATE_MESSAGE msg;
	while (1){
		if (in_use){
			dialog_waitformessage(msg);
			if (!global_ret){
				ret = global_ret;
				break;
			}
		}else{
			global_ret = true;
			in_use = true;
			ret = xconf_yesno (title,intro,help_nil)==MENU_YES;
			global_ret = ret;
			in_use = false;
			dialog_sendmessage (msg);
			break;
		}
	}
	return ret;
}

